@@ -8,7 +8,7 @@ Please find a description of this package at:
 
 Author: Ilan Schnell
 """
-__version__ = '0.5.0'
+__version__ = '0.5.2'
 
 from ._bitarray import _bitarray, bits2bytes, _sysinfo
 
@@ -45,6 +45,7 @@ def _check_codedict(codedict):
         if v.length() == 0:
             raise ValueError("non-empty bitarray expected")
 
+
 class bitarray(_bitarray):
     """bitarray([initial][endian=string])
 
@@ -79,7 +80,8 @@ fromfile, tostring, fromstring, tobytes, frombytes."""
     def fromstring(self, string):
         """fromstring(string)
 
-Append from a string, interpreting the string as machine values."""
+Append from a string, interpreting the string as machine values.
+Deprecated since version 0.4.0, use ``frombytes()`` instead."""
         return self.frombytes(string.encode())
 
     def tostring(self):
@@ -87,7 +89,8 @@ Append from a string, interpreting the string as machine values."""
 
 Return the string representing (machine values) of the bitarray.
 When the length of the bitarray is not a multiple of 8, the few remaining
-bits (1..7) are set to 0."""
+bits (1..7) are set to 0.
+Deprecated since version 0.4.0, use ``tobytes()`` instead."""
         return self.tobytes().decode()
 
     def decode(self, codedict):
@@ -111,7 +114,7 @@ with the corresponding bitarray for each symbols."""
         """search(x[, limit])
 
 Given a bitarray x (or an object which can be converted to a bitarray),
-returns the start positions of x matching self as a list.
+returns the start positions where x matches self as a list.
 The optional argument limits the number of search results to the integer
 specified.  By default, all search results are returned."""
         return self._search(bitarray(x), limit)
@@ -120,7 +123,7 @@ specified.  By default, all search results are returned."""
         """itersearch(x)
 
 Given a bitarray x (or an object which can be converted to a bitarray),
-iterates over the start positions of x matching self."""
+iterates over the start positions where x matches self."""
         x = bitarray(x)
         p = 0
         while 1:
@@ -149,9 +152,9 @@ will be performed to determine return value."""
             return self._search_at(bitarray(other), 0) is not None
 
 
-def test(verbosity=1):
-    """test(verbosity=1)
+def test(verbosity=1, repeat=1):
+    """test(verbosity=1, repeat=1)
 
 Run self-test."""
     from . import test_bitarray
-    return test_bitarray.run(verbosity=verbosity)
+    return test_bitarray.run(verbosity=verbosity, repeat=repeat)
