@@ -263,7 +263,8 @@ insert_n(bitarrayobject *self, idx_t start, idx_t n)
     return 0;
 }
 
-/* set ususet bits to 0, without changing self->nbits, return # bits set */
+/* sets ususet bits to 0, i.e. the ones in the last byte (if any),
+   and return the number of bits set -- self->nbits is unchanged */
 static int
 setunused(bitarrayobject *self)
 {
@@ -355,6 +356,15 @@ setrange(bitarrayobject *self, idx_t start, idx_t stop, int val)
         setbit(self, i, val);
 }
 
+static void
+invert(bitarrayobject *self)
+{
+    Py_ssize_t i;
+
+    for (i = 0; i < Py_SIZE(self); i++)
+        self->ob_item[i] = ~self->ob_item[i];
+}
+
 /* reverse the order of bits in each byte of the buffer */
 static void
 bytereverse(bitarrayobject *self)
@@ -362,15 +372,14 @@ bytereverse(bitarrayobject *self)
     static char trans[256];
     static int setup = 0;
     Py_ssize_t i;
-    int c;
+    unsigned char c;
 
     if (!setup) {
-        /* setup a translation table, which maps each byte to it's
-           reversed: trans = {0, 128, 64, 192, 32, 160, ..., 255}
-        */
+        /* setup translation table, which maps each byte to it's reversed:
+           trans = {0, 128, 64, 192, 32, 160, ..., 255} */
         int j, k;
         for (k = 0; k < 256; k++) {
-            trans[k] = '\0';
+            trans[k] = 0x00;
             for (j = 0; j < 8; j++)
                 if (1 << (7 - j) & k)
                     trans[k] |= 1 << j;
@@ -381,21 +390,10 @@ bytereverse(bitarrayobject *self)
     setunused(self);
     for (i = 0; i < Py_SIZE(self); i++) {
         c = self->ob_item[i];
-        /* in case conversion to int puts a sign in */
-        if (c < 0) c += 256;
         self->ob_item[i] = trans[c];
     }
 }
 
-static void
-invert(bitarrayobject *self)
-{
-    Py_ssize_t i;
-
-    for (i = 0; i < Py_SIZE(self); i++)
-        self->ob_item[i] = ~self->ob_item[i];
-}
-
 /* returns number of 1 bits */
 static idx_t
 count(bitarrayobject *self)
@@ -404,12 +402,11 @@ count(bitarrayobject *self)
     static int setup = 0;
     Py_ssize_t i;
     idx_t res = 0;
-    int c;
+    unsigned char c;
 
     if (!setup) {
-        /* setup a translation table, which maps each byte to it's
-           bit count: trans = {0, 1, 1, 2, 1, 2, 2, 3, 1, ..., 8}
-        */
+        /* setup lookup table, which maps each byte to it's bit count:
+           bitcount = {0, 1, 1, 2, 1, 2, 2, 3, 1, ..., 8} */
         int j, k;
         for (k = 0; k < 256; k++) {
             bitcount[k] = 0;
@@ -422,8 +419,6 @@ count(bitarrayobject *self)
     setunused(self);
     for (i = 0; i < Py_SIZE(self); i++) {
         c = self->ob_item[i];
-        /* in case conversion to int puts a sign in */
-        if (c < 0) c += 256;
         res += bitcount[c];
     }
     return res;
@@ -431,7 +426,7 @@ count(bitarrayobject *self)
 
 /* return index of first occurrence of vi, -1 when x is not in found. */
 static idx_t
-findfirst(bitarrayobject *self, int vi)
+findfirst(bitarrayobject *self, int vi, idx_t start, idx_t stop)
 {
     Py_ssize_t j;
     idx_t i;
@@ -439,19 +434,33 @@ findfirst(bitarrayobject *self, int vi)
 
     if (Py_SIZE(self) == 0)
         return -1;
+    if (start < 0 || start > self->nbits)
+        start = 0;
+    if (stop < 0 || stop > self->nbits)
+        stop = self->nbits;
+    if (start >= stop)
+        return -1;
 
-    c = vi ? 0x00 : 0xff;  /* inverse */
+    if (stop > start + 8) {
+        /* seraching for 1 means: break when byte is not 0x00
+           searching for 0 means: break when byte is not 0xff */
+        c = vi ? 0x00 : 0xff;
 
-    /* skip ahead by checking whole bytes */
-    for (j = 0; j < Py_SIZE(self); j++)
-        if (c ^ self->ob_item[j])
-            break;
+        /* skip ahead by checking whole bytes */
+        for (j = (Py_ssize_t) (start / 8); j < BYTES(stop); j++)
+            if (c ^ self->ob_item[j])
+                break;
 
-    if (j == Py_SIZE(self))
-        j--;
+        if (j == Py_SIZE(self))
+            j--;
+        assert (j >= 0 && j < Py_SIZE(self));
 
-    /* fine grained search within byte */
-    for (i = BITS(j); i < self->nbits; i++)
+        if (start < BITS(j))
+            start = BITS(j);
+    }
+
+    /* fine grained search */
+    for (i = start; i < stop; i++)
         if (GETBIT(self, i) == vi)
             return i;
 
@@ -591,7 +600,7 @@ extend_tuple(bitarrayobject *self, PyObject *tuple)
 */
 enum conv_tp {
     STR_01,    /*  '0' -> 0    '1'  -> 1   no other characters allowed */
-    STR_RAW,   /*  '\0' -> 0   other -> 1                              */
+    STR_RAW,   /*  0x00 -> 0   other -> 1                              */
 };
 
 static int
@@ -674,9 +683,10 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
 
 #ifdef IS_PY3K
     if (PyUnicode_Check(obj)) {                               /* str01 */
-        iter = PyUnicode_AsEncodedString(obj, NULL, NULL);
-        ret = extend_string(self, iter, STR_01);
-        Py_DECREF(iter);
+        PyObject *string;
+        string = PyUnicode_AsEncodedString(obj, NULL, NULL);
+        ret = extend_string(self, string, STR_01);
+        Py_DECREF(string);
         return ret;
     }
 #endif
@@ -757,7 +767,7 @@ slice_GetIndicesEx(PySliceObject *r, idx_t length,
             return -1;
         }
     }
-    defstart = *step < 0 ? length-1 : 0;
+    defstart = *step < 0 ? length - 1 : 0;
     defstop = *step < 0 ? -1 : length;
 
     if (r->start == Py_None) {
@@ -859,16 +869,20 @@ Return number of occurrences of x in the bitarray.  x defaults to True.");
 
 
 static PyObject *
-bitarray_index(bitarrayobject *self, PyObject *v)
+bitarray_index(bitarrayobject *self, PyObject *args)
 {
-    idx_t i;
+    PyObject *x;
+    idx_t i, start = 0, stop = -1;
     long vi;
 
-    vi = PyObject_IsTrue(v);
+    if (!PyArg_ParseTuple(args, "O|LL:index", &x, &start, &stop))
+        return NULL;
+
+    vi = PyObject_IsTrue(x);
     if (vi < 0)
         return NULL;
 
-    i = findfirst(self, vi);
+    i = findfirst(self, vi, start, stop);
     if (i < 0) {
         PyErr_SetString(PyExc_ValueError, "index(x): x not in bitarray");
         return NULL;
@@ -877,7 +891,7 @@ bitarray_index(bitarrayobject *self, PyObject *v)
 }
 
 PyDoc_STRVAR(index_doc,
-"index(x)\n\
+"index(x, [start, [stop]])\n\
 \n\
 Return index of the first occurrence of x in the bitarray.\n\
 It is an error when x does not occur in the bitarray");
@@ -1053,10 +1067,10 @@ Append the value bool(x) to the end of the bitarray.");
 static PyObject *
 bitarray_all(bitarrayobject *self)
 {
-    if (findfirst(self, 0) >= 0)
+    if (findfirst(self, 0, 0, -1) >= 0)
         Py_RETURN_FALSE;
-
-    Py_RETURN_TRUE;
+    else
+        Py_RETURN_TRUE;
 }
 
 PyDoc_STRVAR(all_doc,
@@ -1068,10 +1082,10 @@ Returns True when all bits in the array are True.");
 static PyObject *
 bitarray_any(bitarrayobject *self)
 {
-    if (findfirst(self, 1) >= 0)
+    if (findfirst(self, 1, 0, -1) >= 0)
         Py_RETURN_TRUE;
-
-    Py_RETURN_FALSE;
+    else
+        Py_RETURN_FALSE;
 }
 
 PyDoc_STRVAR(any_doc,
@@ -1123,7 +1137,7 @@ bitarray_reverse(bitarrayobject *self)
         return NULL;
 
 #define tt  ((bitarrayobject *) t)
-    /* copy first half of array into temporary array */
+    /* copy lower half of array into temporary array */
     memcpy(tt->ob_item, self->ob_item, Py_SIZE(tt));
 
     m = self->nbits - 1;
@@ -1383,7 +1397,7 @@ bitarray_fromfile(bitarrayobject *self, PyObject *args)
 #endif
 
 PyDoc_STRVAR(fromfile_doc,
-"fromfile(f [, n])\n\
+"fromfile(f, [n])\n\
 \n\
 Read n bytes from the file object f and append them to the bitarray\n\
 interpreted as machine values.  When n is omitted, as many bytes are\n\
@@ -1509,7 +1523,7 @@ bitarray_frombytes(bitarrayobject *self, PyObject *string)
 PyDoc_STRVAR(frombytes_doc,
 "frombytes(bytes)\n\
 \n\
-Append from a byte string, interpreting the string as machine values.");
+Append from a byte string, interpreted as machine values.");
 
 
 static PyObject *
@@ -1681,7 +1695,7 @@ bitarray_pop(bitarrayobject *self, PyObject *args)
 PyDoc_STRVAR(pop_doc,
 "pop([i])\n\
 \n\
-Return the i-th element and delete it from the bitarray. i defaults to -1.");
+Return the i-th (default last) element and delete it from the bitarray.");
 
 
 static PyObject *
@@ -1694,7 +1708,7 @@ bitarray_remove(bitarrayobject *self, PyObject *v)
     if (vi < 0)
         return NULL;
 
-    i = findfirst(self, vi);
+    i = findfirst(self, vi, 0, -1);
     if (i < 0) {
         PyErr_SetString(PyExc_ValueError, "remove(x): x not in bitarray");
         return NULL;
@@ -1720,10 +1734,8 @@ bitarray_getitem(bitarrayobject *self, PyObject *a)
 
     if (ISINDEX(a)) {
         getIndex(a, &i);
-
         if (i < 0)
             i += self->nbits;
-
         if (i < 0 || i >= self->nbits) {
             PyErr_SetString(PyExc_IndexError, "bitarray index out of range");
             return NULL;
@@ -1790,14 +1802,13 @@ setslice(bitarrayobject *self, PySliceObject *slice, PyObject *v)
 #undef vv
         return 0;
     }
-    else if (PyBool_Check(v)) {
+    if (PyBool_Check(v)) {
         vi = PyObject_IsTrue(v);
         if (vi < 0)
             return -1;
 
         for (i = 0, j = start; i < slicelength; i++, j += step)
             setbit(self, j, vi);
-
         return 0;
     }
     PyErr_SetString(PyExc_IndexError,
@@ -1816,25 +1827,22 @@ bitarray_setitem(bitarrayobject *self, PyObject *args)
 
     if (ISINDEX(a)) {
         getIndex(a, &i);
-
         if (i < 0)
             i += self->nbits;
-
         if (i < 0 || i >= self->nbits) {
             PyErr_SetString(PyExc_IndexError, "bitarray index out of range");
             return NULL;
         }
         set_item(self, i, v);
+        Py_RETURN_NONE;
     }
-    else if (PySlice_Check(a)) {
+    if (PySlice_Check(a)) {
         if (setslice(self, (PySliceObject *) a, v) < 0)
             return NULL;
+        Py_RETURN_NONE;
     }
-    else {
-        PyErr_SetString(PyExc_TypeError, "index or slice expected");
-        return NULL;
-    }
-    Py_RETURN_NONE;
+    PyErr_SetString(PyExc_TypeError, "index or slice expected");
+    return NULL;
 }
 
 static PyObject *
@@ -1844,13 +1852,10 @@ bitarray_delitem(bitarrayobject *self, PyObject *a)
 
     if (ISINDEX(a)) {
         getIndex(a, &i);
-
         if (i < 0)
             i += self->nbits;
-
         if (i < 0 || i >= self->nbits) {
-            PyErr_SetString(PyExc_IndexError,
-                            "bitarray assignment index out of range");
+            PyErr_SetString(PyExc_IndexError, "bitarray index out of range");
             return NULL;
         }
         if (delete_n(self, i, 1) < 0)
@@ -2129,7 +2134,7 @@ bitarray_methods[] = {
      fromfile_doc},
     {"frombytes",    (PyCFunction) bitarray_frombytes,   METH_O,
      frombytes_doc},
-    {"index",        (PyCFunction) bitarray_index,       METH_O,
+    {"index",        (PyCFunction) bitarray_index,       METH_VARARGS,
      index_doc},
     {"insert",       (PyCFunction) bitarray_insert,      METH_VARARGS,
      insert_doc},
@@ -2275,8 +2280,8 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static PyObject *
 richcompare(PyObject *v, PyObject *w, int op)
 {
-    int cmp, k, vi, wi;
-    idx_t i;
+    int cmp, vi, wi;
+    idx_t i, vs, ws;
 
     if (!bitarray_Check(v) || !bitarray_Check(w)) {
         Py_INCREF(Py_NotImplemented);
@@ -2284,65 +2289,55 @@ richcompare(PyObject *v, PyObject *w, int op)
     }
 #define va  ((bitarrayobject *) v)
 #define wa  ((bitarrayobject *) w)
-    if (va->nbits != wa->nbits && (op == Py_EQ || op == Py_NE)) {
-        /* shortcut: if the lengths differ, the bitarrays differ */
+    vs = va->nbits;
+    ws = wa->nbits;
+    if (vs != ws) {
+        /* shortcut for EQ/NE: if sizes differ, the bitarrays differ */
         if (op == Py_EQ)
             Py_RETURN_FALSE;
-
-        Py_RETURN_TRUE;
+        if (op == Py_NE)
+            Py_RETURN_TRUE;
     }
+
     /* to avoid uninitialized warning for some compilers */
     vi = wi = 0;
     /* search for the first index where items are different */
-    k = 1;
-    for (i = 0; i < va->nbits && i < wa->nbits; i++) {
+    for (i = 0; i < vs && i < ws; i++) {
         vi = GETBIT(va, i);
         wi = GETBIT(wa, i);
-        k = (vi == wi);
-        if (k == 0)
-            break;
+        if (vi != wi) {
+            /* we have an item that differs -- first, shortcut for EQ/NE */
+            if (op == Py_EQ)
+                Py_RETURN_FALSE;
+            if (op == Py_NE)
+                Py_RETURN_TRUE;
+            /* compare the final item using the proper operator */
+            switch (op) {
+            case Py_LT: cmp = vi <  wi; break;
+            case Py_LE: cmp = vi <= wi; break;
+            case Py_EQ: cmp = vi == wi; break;
+            case Py_NE: cmp = vi != wi; break;
+            case Py_GT: cmp = vi >  wi; break;
+            case Py_GE: cmp = vi >= wi; break;
+            default: return NULL;  /* cannot happen */
+            }
+            return PyBool_FromLong((long) cmp);
+        }
     }
-    if (k) {
-        /* no more items to compare -- compare sizes */
-        idx_t vs = va->nbits;
-        idx_t ws = wa->nbits;
 #undef va
 #undef wa
-        switch (op) {
-        case Py_LT: cmp = vs <  ws; break;
-        case Py_LE: cmp = vs <= ws; break;
-        case Py_EQ: cmp = vs == ws; break;
-        case Py_NE: cmp = vs != ws; break;
-        case Py_GT: cmp = vs >  ws; break;
-        case Py_GE: cmp = vs >= ws; break;
-        default: return NULL; /* cannot happen */
-        }
-        if (cmp)
-            Py_RETURN_TRUE;
 
-        Py_RETURN_FALSE;
-    }
-    /* We have an item that differs.  First, shortcuts for EQ/NE */
-    if (op == Py_EQ)
-        Py_RETURN_FALSE;
-
-    if (op == Py_NE)
-        Py_RETURN_TRUE;
-
-    /* compare the final item using the proper operator */
+    /* no more items to compare -- compare sizes */
     switch (op) {
-    case Py_LT: cmp = vi <  wi; break;
-    case Py_LE: cmp = vi <= wi; break;
-    case Py_EQ: cmp = vi == wi; break;
-    case Py_NE: cmp = vi != wi; break;
-    case Py_GT: cmp = vi >  wi; break;
-    case Py_GE: cmp = vi >= wi; break;
-    default: return NULL; /* cannot happen */
+    case Py_LT: cmp = vs <  ws; break;
+    case Py_LE: cmp = vs <= ws; break;
+    case Py_EQ: cmp = vs == ws; break;
+    case Py_NE: cmp = vs != ws; break;
+    case Py_GT: cmp = vs >  ws; break;
+    case Py_GE: cmp = vs >= ws; break;
+    default: return NULL;  /* cannot happen */
     }
-    if (cmp)
-        Py_RETURN_TRUE;
-
-    Py_RETURN_FALSE;
+    return PyBool_FromLong((long) cmp);
 }
 
 /************************** Bitarray Iterator **************************/
@@ -2528,7 +2523,12 @@ sysinfo(void)
 }
 
 PyDoc_STRVAR(sysinfo_doc,
-"_sysinfo()");
+"_sysinfo()\n\
+\n\
+tuple(sizeof(void *),\n\
+      sizeof(size_t),\n\
+      sizeof(Py_ssize_t),\n\
+      PY_SSIZE_T_MAX)");
 
 
 static PyMethodDef module_functions[] = {
