@@ -18,12 +18,7 @@ else:
     from cStringIO import StringIO
 
 
-if __name__ == '__main__':
-    from __init__ import bitarray, bits2bytes
-    repr_type = "<class '__init__.bitarray'>"
-else:
-    from bitarray import bitarray, bits2bytes
-    repr_type = "<class 'bitarray.bitarray'>"
+from bitarray import bitarray, bits2bytes, __version__
 
 
 tests = []
@@ -60,7 +55,7 @@ class Util(object):
         return getIndicesEx(r, length)[-1]
 
     def check_obj(self, a):
-        self.assertEqual(repr(type(a)), repr_type)
+        self.assertEqual(repr(type(a)), "<class 'bitarray.bitarray'>")
         unused = 8 * a.buffer_info()[1] - len(a)
         self.assert_(0 <= unused < 8)
         self.assertEqual(unused, a.buffer_info()[3])
@@ -514,6 +509,11 @@ tests.append(SliceTests)
 
 class MiscTests(unittest.TestCase, Util):
 
+    def test_instancecheck(self):
+        a = bitarray('011')
+        self.assertTrue(isinstance(a, bitarray))
+        self.assertFalse(isinstance(a, str))
+
     def test_booleanness(self):
         self.assertEqual(bool(bitarray('')), False)
         self.assertEqual(bool(bitarray('0')), True)
@@ -1147,13 +1147,13 @@ class MethodTests(unittest.TestCase, Util):
         for i in (True, False, 1, 0):
             self.assertRaises(ValueError, a.index, i)
 
-        a = bitarray(100*[False])
+        a = bitarray(100 * [False])
         self.assertRaises(ValueError, a.index, True)
-        a[20] = a[27] = 54
+        a[20] = a[27] = 1
         self.assertEqual(a.index(42), 20)
         self.assertEqual(a.index(0), 0)
 
-        a = bitarray(200*[True])
+        a = bitarray(200 * [True])
         self.assertRaises(ValueError, a.index, False)
         a[173] = a[187] = 0
         self.assertEqual(a.index(False), 173)
@@ -1172,6 +1172,34 @@ class MethodTests(unittest.TestCase, Util):
                 a[m] = 0
                 self.assertEqual(a.index(0), m)
 
+    def test_index2(self):
+        a = bitarray('00001000' '00000000' '0010000')
+        self.assertEqual(a.index(1), 4)
+        self.assertEqual(a.index(1, 1), 4)
+        self.assertEqual(a.index(0, 4), 5)
+        self.assertEqual(a.index(1, 5), 18)
+        self.assertRaises(ValueError, a.index, 1, 5, 18)
+        self.assertRaises(ValueError, a.index, 1, 19)
+
+    def test_index3(self):
+        a = bitarray(2000)
+        a.setall(0)
+        for _ in range(3):
+            a[randint(0, 1999)] = 1
+        aa = a.tolist()
+        for _ in range(100):
+            start = randint(0, 2000)
+            stop = randint(0, 2000)
+            try:
+                res1 = a.index(1, start, stop)
+            except ValueError:
+                res1 = None
+            try:
+                res2 = aa.index(1, start, stop)
+            except ValueError:
+                res2 = None
+            self.assertEqual(res1, res2)
+
 
     def test_count(self):
         a = bitarray('10011')
@@ -1182,6 +1210,11 @@ class MethodTests(unittest.TestCase, Util):
         self.assertEqual(a.count(0), 2)
         self.assertRaises(TypeError, a.count, 'A')
 
+        for i in range(0, 256):
+            a = bitarray()
+            a.frombytes(to_bytes(chr(i)))
+            self.assertEqual(a.count(), a.to01().count('1'))
+
         for a in self.randombitarrays():
             self.assertEqual(a.count(), a.count(1))
             self.assertEqual(a.count(1), a.to01().count('1'))
@@ -1329,17 +1362,16 @@ class MethodTests(unittest.TestCase, Util):
     def test_reverse(self):
         self.assertRaises(TypeError, bitarray().reverse, 42)
 
-        a = bitarray()
-        a.reverse()
-        self.assertEQUAL(a, bitarray())
-
-        a = bitarray('1001111')
-        a.reverse()
-        self.assertEQUAL(a, bitarray('1111001'))
-
-        a = bitarray('11111000011')
-        a.reverse()
-        self.assertEQUAL(a, bitarray('11000011111'))
+        for x, y in [('', ''), ('1', '1'), ('10', '01'), ('001', '100'),
+                     ('1110', '0111'), ('11100', '00111'),
+                     ('011000', '000110'), ('1101100', '0011011'),
+                     ('11110000', '00001111'),
+                     ('11111000011', '11000011111'),
+                     ('11011111' '00100000' '000111',
+                      '111000' '00000100' '11111011')]:
+            a = bitarray(x)
+            a.reverse()
+            self.assertEQUAL(a, bitarray(y))
 
         for a in self.randombitarrays():
             aa = a.tolist()
@@ -1373,10 +1405,11 @@ class MethodTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, a.remove, 0)
 
         a = bitarray('1010110')
-        a.remove(False);        self.assertEqual(a, bitarray('110110'))
-        a.remove(True);         self.assertEqual(a, bitarray('10110'))
-        a.remove(1);            self.assertEqual(a, bitarray('0110'))
-        a.remove(0);            self.assertEqual(a, bitarray('110'))
+        for val, res in [(False, '110110'), (True, '10110'),
+                         (1, '0110'), (1, '010'), (0, '10'),
+                         (0, '1'), (1, '')]:
+            a.remove(val)
+            self.assertEQUAL(a, bitarray(res))
 
         a = bitarray('0010011')
         b = a
@@ -1386,7 +1419,16 @@ class MethodTests(unittest.TestCase, Util):
 
 
     def test_pop(self):
-        a = bitarray()
+        for x, n, r, y in [('1', 0, True, ''),
+                           ('0', -1, False, ''),
+                           ('0011100', 3, True, '001100')]:
+            a = bitarray(x)
+            self.assertEqual(a.pop(n), r)
+            self.assertEqual(a, bitarray(y))
+
+        a = bitarray('01')
+        self.assertEqual(a.pop(), True)
+        self.assertEqual(a.pop(), False)
         self.assertRaises(IndexError, a.pop)
 
         for a in self.randombitarrays():
@@ -1406,6 +1448,8 @@ class MethodTests(unittest.TestCase, Util):
             n = randint(-len(a), len(a)-1)
             aa = a.tolist()
             self.assertEqual(a.pop(n), aa[n])
+            aa.pop(n)
+            self.assertEqual(a, bitarray(aa))
             self.check_obj(a)
 
 
@@ -1918,53 +1962,19 @@ tests.append(PrefixCodeTests)
 
 # ---------------------------------------------------------------------------
 
-def pages():
-    if sys.platform != 'linux2':
-        return 0
-    dat = open('/proc/%i/statm' % os.getpid()).read()
-    return int(dat.split()[0])
+def run(verbosity=1, repeat=1):
+    print('bitarray is installed in: ' + os.path.dirname(__file__))
+    print('bitarray version: ' + __version__)
+    print(sys.version)
 
-
-def check_memory_leaks(verbosity):
     suite = unittest.TestSuite()
     for cls in tests:
-        suite.addTest(unittest.makeSuite(cls))
-
-    logfile = 'pages.log'
-    if os.path.isfile(logfile):
-        os.unlink(logfile)
-
-    i = 0
-    runner = unittest.TextTestRunner(verbosity=verbosity)
-    while True:
-        print('Run: %d' % i)
-        r = runner.run(suite)
-        if i % 1 == 0:
-            fo = open(logfile, 'a')
-            fo.write('%10i %r %10i\n' % (i, r.wasSuccessful(), pages()))
-            fo.close()
-        i += 1
-
-
-def run(verbosity, chk_mem_leaks=False):
-    suite = unittest.TestSuite()
-    for cls in tests:
-        suite.addTest(unittest.makeSuite(cls))
+        for _ in range(repeat):
+            suite.addTest(unittest.makeSuite(cls))
 
     runner = unittest.TextTestRunner(verbosity=verbosity)
-
     return runner.run(suite)
 
 
 if __name__ == '__main__':
-    verbosity = 2 if 'v' in sys.argv else 1
-    if 'm' in sys.argv:
-        check_memory_leaks(verbosity)
-    else:
-        run(verbosity)
-
-else:
-    from bitarray import __version__
-    print('bitarray is installed in: ' + os.path.dirname(__file__))
-    print('bitarray version: ' + __version__)
-    print(sys.version)
+    run()
